/*
 * Part of network management sources. It is used in daemon that
 * checks which network devices are online on local subnets.
 *
 * Copyright (c) 2006 JAS
 *
 * Author: Petr Vokac <vokac@kmlinux.fjfi.cvut.cz>
 *
 * $Id: ArpData.hpp 13 2006-04-27 01:36:19Z vokac $
 */
#ifndef _ARP_DATA_HPP
#define _ARP_DATA_HPP

#include <time.h>
#include <pthread.h>
#include <map>
#include <vector>


using namespace std;


typedef struct _DataInfo {
  time_t first, last, last_probe;
  unsigned int ip;
  unsigned char mac[6];
} DataInfo;


class ArpData 
{
private:

  typedef map<unsigned int, DataInfo *> _data_map;
  _data_map _data;

  pthread_mutex_t _mutex;

public:

  const static int NEW_RECORD = 1;
  const static int UPDATE_RECORD_TIME = 2;
  const static int UPDATE_RECORD_LAST = 3;
  const static int UPDATE_RECORD_MAC = 4;

  // constructor
  ArpData();

  // destructor
  ~ArpData();

  // debug dump
  char *dump(unsigned int lower = 0, unsigned int upper = -1);

  // add data
  int add(unsigned int ip, const unsigned char mac[6] = NULL, time_t tm = 0);
  int add(const u_int8_t ip[4], const u_int8_t mac[6] = NULL, time_t tm = 0);
  int add(char *ip, char *mac = NULL, time_t tm = 0);
  
  // clean data
  void clean(time_t from = 0, time_t to = 0);

  // list all stored data
  // XXX: <from, to> time is not used!!!
  DataInfo* list(time_t from = 0, time_t to = 0);

  // list all stored IP addresses
  // XXX: <from, to> time is not used!!!
  vector<unsigned int>* listIP(time_t from = 0, time_t to = 0);

  // get data by IP address
  DataInfo* getByIP(unsigned int ip);
  DataInfo* getByIP(char *ip);
  
  // get data by MAC address
  DataInfo* getByMAC(char mac[6]);

  // convert string reprezentation of IP address to int
  static unsigned int convertIP(char *ip);
  static unsigned char* convertMAC(char *mac);

};

#endif
