/*
 * Part of network management sources. It is used in daemon that
 * checks which network devices are online on local subnets.
 *
 * Copyright (c) 2006 JAS
 *
 * Author: Petr Vokac <vokac@kmlinux.fjfi.cvut.cz>
 *
 * $Id: Capture.hpp 7 2006-04-26 09:57:52Z vokac $
 */
#ifndef _CAPTURE_HPP
#define _CAPTURE_HPP

#include <syslog.h>
#include <pcap.h>
#include <pthread.h>
#include <map>

#include "ArpData.hpp"


using namespace std;


void *init_pcap_loop(void *data);
void got_packet(u_char *args, const struct pcap_pkthdr *header, const u_char *packet);


struct ltstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }
};


class Capture 
{
public:

  typedef pair<pcap_t *, pthread_t *> _pcap_thread;
  typedef map<const char *, _pcap_thread*, ltstr > _data_map;

  static const int maxPacketLength = 1518;

private:

  _data_map _data;
  ArpData *_arp_data;

  pthread_mutex_t _capture_start_mutex;
  pthread_cond_t _capture_start_mutex_cond;
  bool _capture_start_ok;

public:

  // constructor
  // @param arp_data reference to ArpData object
  Capture(ArpData *arp_data);

  // destructor
  ~Capture();

  // Add new interface for listening network traffic
  // @param name interface name
  // @return success(0)/failure(-1)/warning(1)
  int addInterface(const char *name);

  // Stop listening netword traffic on defined interface
  // @param name interface name
  // @return success(0)/failure(-1)/warning(1)
  int removeInterface(const char *name);
};


typedef struct _capture_data {
  char *name;
  Capture::_pcap_thread *pt;
  bool *_capture_start_ok;
  pthread_mutex_t *_capture_start_mutex;
  pthread_cond_t *_capture_start_mutex_cond;
  ArpData *_arp_data;
} capture_data;

#endif
