/*
 * Part of network management sources. It is used in daemon that
 * checks which network devices are online on local subnets.
 *
 * Copyright (c) 2006 JAS
 *
 * Author: Petr Vokac <vokac@kmlinux.fjfi.cvut.cz>
 *
 * $Id: Command.hpp 13 2006-04-27 01:36:19Z vokac $
 */
#ifndef _COMMAND_HPP
#define _COMMAND_HPP

#include <syslog.h>
#include <pcap.h>
#include <pthread.h>

#include <openssl/ssl.h>
#include <openssl/err.h>

#include <exception>

#include "ArpData.hpp"
#include "Request.hpp"


using namespace std;


void *connection(void* data);


typedef struct _command_conn_data 
{
  int port;
  int min_conn;
  int max_conn;

  char *ca_file;
  char *crl_file;
  char *cert_file;
  char *key_file;

  char *user;
  char *pass;
} command_conn_data;

typedef struct _my_listen_thread
{
  bool in_use;
  int socket_id;
  SSL *ssl;

  pthread_t thread;
  pthread_mutex_t *_thread_pool_mutex;

  ArpData *data;
  Request *req;

  char *user;
  char *pass;
} my_listen_thread;


class Command
{

private:

  ArpData *_data;
  Request *_req;

  pthread_mutex_t _thread_pool_mutex;

  int _port;
  int _min_conn;
  int _max_conn;
  int _sock;

  char *_ca_file;
  char *_crl_file;
  char *_cert_file;
  char *_key_file;

  char *_user;
  char *_pass;

  SSL_CTX *_ctx;  

  SSL_CTX* initServerCTX();
  void loadCertificates(SSL_CTX* ctx, char* certFile, char* keyFile);

public:

  // constructor
  // @param conn_data connection properties
  // @param data reference to ArpData object
  // @param req reference to Request object
  Command(command_conn_data *conn_data, ArpData *data, Request *req);

  // destructor
  ~Command();
  
  // run command listener
  void run();

};

#endif
