/*
 * Part of network management sources. It is used in daemon that
 * checks which network devices are online on local subnets.
 *
 * Copyright (c) 2006 JAS
 *
 * Author: Petr Vokac <vokac@kmlinux.fjfi.cvut.cz>
 *
 * $Id: Request.hpp 13 2006-04-27 01:36:19Z vokac $
 */
#ifndef _REQUEST_HPP
#define _REQUEST_HPP

#include <stdio.h>
#include <linux/if_packet.h>
#include <vector>

#include "ArpData.hpp"


using namespace std;


class Request 
{

private:

  ArpData *_data;

  int _sock;
  struct sockaddr_ll _me, _he;
  unsigned char _buf[255];
  int _buf_len;
  unsigned char *_buf_pos;

public:

  // constructor
  // @param data reference to ArpData (to update scan info)
  Request(ArpData *data);

  // destructor
  ~Request();

  // send new arp request for ip/name
  // @param ip int representation of ip address
  // @param exp time to determine "expired" records
  // @param expNull time to determine "expired" records with mac 00:00:00:00:00:00
  void send(unsigned int ip, int exp = 0, int expNull = 0);
  
  // send new arp request for range IP addresses
  // @param from_ip lower bound of IP addresses
  // @param to_ip upper bound of IP addresses
  // @param exp time to determine "expired" records
  // @param expNull time to determine "expired" records with mac 00:00:00:00:00:00
  void scan(unsigned int from_ip, unsigned int to_ip, int exp = 0, int expNull = 0);

  // send new arp request for list of IP addresses
  // @param ip vector of IP addresses to scan
  // @param exp time to determine "expired" records
  // @param expNull time to determine "expired" records with mac 00:00:00:00:00:00
  void scan(vector<unsigned int> *ip, int exp = 0, int expNull = 0);
  
};

#endif
